% esempio di identificazione di un modello ARX(1,1)
% facendo uso del System Identification Toolbox
% testato in R2012a
%
% corso "Sistemi Dinamici"
% a.a. 2019/2020
% G. Fenu

clc
clear all

% sistema vero
% y(t) = a1 * y(t-1) +b1*u(t-1) +e(t)

% parametrizazione vera
a1_o=0.45;
b1_o=0.7;
theta_o =[a1_o;b1_o];

A = [1 -a1_o]; % polinomio A(z)
B = [0 b1_o];  % polinomio B(z)
lambda2 = 0.1; % la varianza del rumore
% creo il sistema vero
m0 = idpoly(A,B,[],[],[], lambda2 );
% inserisco anche la varianza del rumore


% lo simulo con dei segnali
N1 = 200; % uso 200 campioni
u = iddata([],idinput(N1,'rbs'));
% ingresso esogeno di tipo "segnale binario casuale"
% rbs <-> random binary signal

optSim = simOptions('AddNoise',true); 
% il rumore viene generato all'interno del simulatore; come default questa
% opportunita' e' disabilitata, quindi se usassi la sintassi
% sim(m0, u)
% simulerei solo la parte deterministica, senza il contributo di rumore
y = sim(m0, u, optSim); 
% cosi' viene inserito rumore, con valor atteso nullo e varianza assegnata

% ordine di eccitazione?
pexcit(u) % ordine di persistente eccitazione

% ora identifico
z = [y,u]; % accorpo i dati
m = arx(z,[1 1 1]);
% M = ARX(DATA,ORDERS)
% ORDERS = [na nb nk], the orders and delays of the above model

% ora estraggo il vettore dei parametri e la sua incertezza
[Aid, Bid, dAid, dBid] = arxdata(m);
% Aid, Bid <-> coefficienti dei polinomi A(z), B(z)
% dAid, dBid <-> deviazione standard - misura della incertezza per ciascun
% parametro di Aid, Bid
a1_e = -Aid(2); b1_e = Bid(2);

da1_e = 3*dAid(2);  
db1_e = 3*dBid(2);
% distrib. gaussiana -- con prob. 0.99 valore effettivo in +\- 3 std

% rappresento graficamente
fh = figure('Name','prestazioni della stima al variare di N');
plot(a1_o, b1_o, 'or'); hold on; grid on
xlabel('a');ylabel('b')

% prima stima
drawFillEllispe2D(a1_e, b1_e, ...
                    da1_e, db1_e, 0, ...
                    [0.4, 0.4,0.4], [0.7, 0.7,0.7]);
                
plot(a1_e, b1_e, 's', 'MarkerSize', 10, ...
    'MarkerEdgeColor', [0, 0, 1],...
    'MarkerFaceColor', [0, 0, 1]);
hold on; grid on

plot(a1_o, b1_o, 'o','MarkerSize', 8, ...
    'MarkerEdgeColor', [1, 0, 0],...
    'MarkerFaceColor', [1, 0, 0]); 

set(gca, 'FontSize', 14, 'FontWeight', 'bold');


% lo simulo con dei segnali
N2 = 20000; % uso 20000 campioni
u = iddata([],idinput(N2,'rbs'));% ingresso esogeno
% come in precedenza abilito la generazione di rumore nel modello in fase
% di simulazione
optSim = simOptions('AddNoise',true); 
y = sim(m0, u, optSim); 
% anche stavolta la simulazione tien conto del rumore

% ordine di eccitazione?
pexcit(u) % ordine di persistente eccitazione

% ora identifico
z = [y,u]; % accorpo i dati
m2 = arx(z,[1 1 1]);
% M = ARX(DATA,ORDERS)
% ORDERS = [na nb nk], the orders and delays of the above model

% ora estraggo il vettore dei parametri e la sua incertezza
[Aid2, Bid2, dAid2, dBid2] = arxdata(m2);
a1_e2 = -Aid2(2); 
b1_e2 = Bid2(2);

da1_e2 = 3*dAid2(2);
db1_e2 = 3*dBid2(2);

% rapresento graficamente
figure(fh);

% seconda stima
% notare come la stima si avvicini al "valore vero" e come si restringe il
% box che misura l'incertezza
drawFillEllispe2D(a1_e2, b1_e2, ...
                    da1_e2, db1_e2, 0, ...
                    [1, 0.1,0.1], [0.85, 0.8,0.8]);
                
plot(a1_e2, b1_e2, 'd', 'MarkerSize', 8, ...
    'MarkerEdgeColor', [0, 1, 1],...
    'MarkerFaceColor', [0, 1, 1]);
plot(a1_o, b1_o, 'o','MarkerSize', 8, ...
    'MarkerEdgeColor', [1, 0, 0],...
    'MarkerFaceColor', [1, 0, 0]); 

